@@ -57,12 +57,13 @@ sympy_mappings = {
 }
 
 def pysr(X=None, y=None, weights=None,
+            binary_operators=["plus", "mult"],
+            unary_operators=["cos", "exp", "sin"],
             procs=4,
+            loss='L2DistLoss()',
             populations=None,
             niterations=100,
             ncyclesperiteration=300,
-            binary_operators=["plus", "mult"],
-            unary_operators=["cos", "exp", "sin"],
             alpha=0.1,
             annealing=True,
             fractionReplaced=0.10,
@@ -116,16 +117,28 @@ def pysr(X=None, y=None, weights=None,
     :param y: np.ndarray, 1D array. Rows are examples.
     :param weights: np.ndarray, 1D array. Each row is how to weight the
         mean-square-error loss on weights.
+    :param binary_operators: list, List of strings giving the binary operators
+        in Julia's Base.
+    :param unary_operators: list, Same but for operators taking a single scalar.
     :param procs: int, Number of processes (=number of populations running).
+    :param loss: str, String of Julia code specifying the loss function.
+        Can either be a loss from LossFunctions.jl, or your own
+        loss written as a function. Examples of custom written losses
+        include: `myloss(x, y) = abs(x-y)` for non-weighted, or 
+        `myloss(x, y, w) = w*abs(x-y)` for weighted.
+        Among the included losses, these are as follows. Regression:
+        `LPDistLoss{P}()`, `L1DistLoss()`, `L2DistLoss()` (mean square),
+        `LogitDistLoss()`, `HuberLoss(d)`, `L1EpsilonInsLoss(ϵ)`,
+        `L2EpsilonInsLoss(ϵ)`, `PeriodicLoss(c)`, `QuantileLoss(τ)`.
+        Classification: `ZeroOneLoss()`, `PerceptronLoss()`, `L1HingeLoss()`,
+        `SmoothedL1HingeLoss(γ)`, `ModifiedHuberLoss()`, `L2MarginLoss()`,
+        `ExpLoss()`, `SigmoidLoss()`, `DWDMarginLoss(q)`.
     :param populations: int, Number of populations running; by default=procs.
     :param niterations: int, Number of iterations of the algorithm to run. The best
         equations are printed, and migrate between populations, at the
         end of each.
     :param ncyclesperiteration: int, Number of total mutations to run, per 10
         samples of the population, per iteration.
-    :param binary_operators: list, List of strings giving the binary operators
-        in Julia's Base, or in `operator.jl`.
-    :param unary_operators: list, Same but for operators taking a single `Float32`.
     :param alpha: float, Initial temperature.
     :param annealing: bool, Whether to use annealing. You should (and it is default).
     :param fractionReplaced: float, How much of population to replace with migrating
@@ -262,7 +275,7 @@ def pysr(X=None, y=None, weights=None,
                  weightSimplify=weightSimplify,
                  constraints=constraints,
                  extra_sympy_mappings=extra_sympy_mappings,
-                 julia_project=julia_project)
+                 julia_project=julia_project, loss=loss)
 
     kwargs = {**_set_paths(tempdir), **kwargs}
 
@@ -310,11 +323,12 @@ def _final_pysr_process(julia_optimization, runfile_filename, timeout, **kwargs)
     ]
     if timeout is not None:
         command = [f'timeout', f'{timeout}'] + command
-    _cmd_runner(command)
+    _cmd_runner(command, **kwargs)
 
-def _cmd_runner(command):
-    print("Running on", ' '.join(command))
-    process = subprocess.Popen(command, stdout=subprocess.PIPE, bufsize=1)
+def _cmd_runner(command, **kwargs):
+    if kwargs['verbosity'] > 0:
+        print("Running on", ' '.join(command))
+    process = subprocess.Popen(command, stdout=subprocess.PIPE, bufsize=-1)
     try:
         while True:
             line = process.stdout.readline()
@@ -354,7 +368,7 @@ def _create_julia_files(dataset_filename, def_datasets,  hyperparam_filename, de
         if len(variable_names) == 0:
             varMap = "[" + ",".join([f'"x{i}"' for i in range(X.shape[1])]) + "]"
         else:
-            varMap = "[" + ",".join(variable_names) + "]"
+            varMap = "[" + ",".join(['"' + vname + '"' for vname in variable_names]) + "]"
 
         if weights is not None:
             print(f'EquationSearch(X, y, weights=weights, niterations={niterations:d}, varMap={varMap}, options=options, numprocs={procs})', file=f)
@@ -364,8 +378,8 @@ def _create_julia_files(dataset_filename, def_datasets,  hyperparam_filename, de
 
 def _make_datasets_julia_str(X, X_filename, weights, weights_filename, y, y_filename, **kwargs):
     def_datasets = """using DelimitedFiles"""
-    np.savetxt(X_filename, X, delimiter=',')
-    np.savetxt(y_filename, y.reshape(-1, 1), delimiter=',')
+    np.savetxt(X_filename, X.astype(np.float32), delimiter=',')
+    np.savetxt(y_filename, y.reshape(-1, 1).astype(np.float32), delimiter=',')
     if weights is not None:
         np.savetxt(weights_filename, weights.reshape(-1, 1), delimiter=',')
     def_datasets += f"""
@@ -382,7 +396,7 @@ def _make_hyperparams_julia_str(X, alpha, annealing, batchSize, batching, binary
                                parsimony, perturbationFactor, populations, procs, shouldOptimizeConstants,
                                unary_operators, useFrequency, use_custom_variable_names,
                                variable_names, warmupMaxsize, weightAddNode,
-                               ncyclesperiteration, fractionReplaced, topn, verbosity,
+                               ncyclesperiteration, fractionReplaced, topn, verbosity, loss,
                                weightDeleteNode, weightDoNothing, weightInsertNode, weightMutateConstant,
                                weightMutateOperator, weightRandomize, weightSimplify, weights, **kwargs):
     def tuple_fix(ops):
@@ -410,11 +424,13 @@ greater=SymbolicRegression.greater
 relu=SymbolicRegression.relu
 logical_or=SymbolicRegression.logical_or
 logical_and=SymbolicRegression.logical_and
+_custom_loss = {loss}
 
 options = SymbolicRegression.Options(binary_operators={'(' + tuple_fix(binary_operators) + ')'},
 unary_operators={'(' + tuple_fix(unary_operators) + ')'},
 {constraints_str}
 parsimony={parsimony:f}f0,
+loss=_custom_loss,
 alpha={alpha:f}f0,
 maxsize={maxsize:d},
 maxdepth={maxdepth:d},
@@ -449,9 +465,6 @@ topn={topn:d},
 verbosity=round(Int32, {verbosity:f})
 """
 
-    if use_custom_variable_names:
-        def_hyperparams += f""",
-    varMap = {'["' + '", "'.join(variable_names) + '"]'}"""
     def_hyperparams += '\n)'
     return def_hyperparams
 
